/// <reference types="bn.js" />
import { PublicKey } from "@solana/web3.js";
import { TransactionBuilder } from "@orca-so/common-sdk";
import { ConfigOptionParams, MarketStateData, EscrowContext, KycCertData, PDA, OfferQuotes, EscrowFetcher } from "..";
import { BN } from "@project-serum/anchor";
export declare class MarketClient {
    ctx: EscrowContext;
    escrowFetcher: EscrowFetcher;
    publicKey: PublicKey;
    data: MarketStateData;
    pda: PDA;
    tokenMint: PublicKey;
    constructor(ctx: EscrowContext, publicKey: PublicKey, tokenMint: PublicKey, pda: PDA, data: MarketStateData);
    static getMarket(ctx: EscrowContext, configId: string, marketId: string, tokenMint: PublicKey): Promise<MarketClient>;
    setTradeConfigAuthority(deployerPubkey: PublicKey, newTradeConfigAuthority: PublicKey): Promise<TransactionBuilder>;
    setAmlPolicyAuthority(deployerPubkey: PublicKey, newAmlPolicyAuthority: PublicKey): Promise<TransactionBuilder>;
    addArbitratorCert(tradeConfigAuthority: PublicKey, arbitratorAuthority: PublicKey): Promise<TransactionBuilder>;
    addTokenMetadata(tradeConfigAuthority: PublicKey, quota30Days: BN[], quota24Hours: BN[]): Promise<TransactionBuilder>;
    addTokenOracleMetadata(tradeConfigAuthority: PublicKey, renecOraclePubkey: PublicKey, quota30Days: BN[], quota24Hours: BN[]): Promise<TransactionBuilder>;
    setRenecRate(tradeConfigAuthority: PublicKey, renecRate: BN): Promise<TransactionBuilder>;
    setUsdRate(tradeConfigAuthority: PublicKey, usdRate: BN): Promise<TransactionBuilder>;
    setMinmumOfferSize(tradeConfigAuthority: PublicKey, minimumOfferSize: BN): Promise<TransactionBuilder>;
    removeArbitratorCert(tradeConfigAuthority: PublicKey, arbitratorAuthority: PublicKey): Promise<TransactionBuilder>;
    updateMarketState(tradeConfigAuthority: PublicKey, option: ConfigOptionParams): Promise<TransactionBuilder>;
    setKycCert(amlPolicyAuthority: PublicKey, user: PublicKey, level: number): Promise<TransactionBuilder>;
    getKycCert(user: PublicKey): Promise<KycCertData>;
    isArbitrator(arbitratorAuthority: PublicKey): Promise<boolean>;
    refresh(): Promise<MarketClient>;
    getOffersQuotesMatchingEngine(user: PublicKey, totalTradeAmount: BN, bondType?: object): Promise<OfferQuotes | null>;
    private buildMatchingEngineTransaction;
    matchingEngine(user: PublicKey, totalTradeAmount: BN, offerQuotes: OfferQuotes, useBondless: boolean, takerBondType: object, buyOfferAccounts?: PublicKey[]): Promise<{
        txs: TransactionBuilder[];
        ids: BN[][];
    }>;
    requestCreateTrade(sellOfferId: BN, user: PublicKey, tradeAmount: BN, useBondless: boolean, takerBondType: object, buyOfferAccount?: PublicKey): Promise<{
        tx: TransactionBuilder;
        id: BN;
    }>;
}
