"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PDA = void 0;
const anchor = __importStar(require("@project-serum/anchor"));
const CONFIG_SEED = "config";
const MARKET_SEED = "market";
const MARKET_TOKEN_VAULT = "market_token_vault";
const MARKET_NATIVE_VAULT = "market_native_vault";
const TRADE_SEED = "trade";
const OFFER_SEED = "offer";
const ARBITRATOR_CERT_SEED = "arbitrator_cert";
const KYC_CERT_SEED = "kyc_cert";
const EVIDENCE_ACCOUNT_SEED = "evidence_account";
class PDA {
    constructor(programId, configId, marketId) {
        this.getConfigPDA = () => {
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([anchor.utils.bytes.utf8.encode(CONFIG_SEED), anchor.utils.bytes.utf8.encode(this.configId)], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getMarketPDA = (tokenMint) => {
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([
                anchor.utils.bytes.utf8.encode(MARKET_SEED),
                anchor.utils.bytes.utf8.encode(this.marketId),
                tokenMint.toBuffer(),
            ], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getMarketTokenVaultPDA = (tokenMint) => {
            const marketPDA = this.getMarketPDA(tokenMint);
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([anchor.utils.bytes.utf8.encode(MARKET_TOKEN_VAULT), marketPDA.key.toBuffer()], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getMarketNativeVaultPDA = (tokenMint) => {
            const marketPDA = this.getMarketPDA(tokenMint);
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([anchor.utils.bytes.utf8.encode(MARKET_NATIVE_VAULT), marketPDA.key.toBuffer()], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getOfferPDA = (offerID) => {
            const configPubkey = this.getConfigPDA().key;
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([
                anchor.utils.bytes.utf8.encode(OFFER_SEED),
                configPubkey.toBuffer(),
                offerID.toArrayLike(Buffer, "le", 8),
            ], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getTradePDA = (tradeID) => {
            const configPubkey = this.getConfigPDA().key;
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([
                anchor.utils.bytes.utf8.encode(TRADE_SEED),
                configPubkey.toBuffer(),
                tradeID.toArrayLike(Buffer, "le", 8),
            ], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getArbitratorCertPDA = (arbitratorAuthority, tokenMint) => {
            const marketPDA = this.getMarketPDA(tokenMint);
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([
                anchor.utils.bytes.utf8.encode(ARBITRATOR_CERT_SEED),
                marketPDA.key.toBuffer(),
                arbitratorAuthority.toBuffer(),
            ], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getKycCertPDA = (user, tokenMint) => {
            const marketPDA = this.getMarketPDA(tokenMint);
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([
                anchor.utils.bytes.utf8.encode(KYC_CERT_SEED),
                marketPDA.key.toBuffer(),
                tokenMint.toBuffer(),
                user.toBuffer(),
            ], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.getEvidencePDA = (user, trade) => {
            const [pda, bump] = anchor.web3.PublicKey.findProgramAddressSync([anchor.utils.bytes.utf8.encode(EVIDENCE_ACCOUNT_SEED), user.toBuffer(), trade.toBuffer()], this.programId);
            return {
                key: pda,
                bump: bump,
            };
        };
        this.programId = programId;
        this.marketId = marketId;
        this.configId = configId;
    }
}
exports.PDA = PDA;
